@@ -30,8 +30,12 @@
 /* For XExtSetCloseDisplay */
 #include <X11/Xlibint.h>
 
+#include "pango-engine-private.h"
 #include "pango-fontmap.h"
 #include "pango-utils.h"
+
+#undef PANGO_DISABLE_DEPRECATED
+
 #include "pangox-private.h"
 
 typedef struct _PangoXFamily       PangoXFamily;
@@ -216,6 +220,7 @@ pango_x_font_map_class_init (PangoFontMapClass *class)
   object_class->finalize = pango_x_font_map_finalize;
   class->load_font = pango_x_font_map_load_font;
   class->list_families = pango_x_font_map_list_families;
+  class->shape_engine_type = PANGO_RENDER_TYPE_X;
 }
 
 /*
@@ -287,7 +292,7 @@ pango_x_font_map_for_display (Display *display)
       tmp_list = tmp_list->next;
     }
 
-  xfontmap = (PangoXFontMap *)g_type_create_instance (PANGO_TYPE_X_FONT_MAP);
+  xfontmap = g_object_new (PANGO_TYPE_X_FONT_MAP, NULL);
   
   xfontmap->display = display;
   xfontmap->font_cache = pango_x_font_cache_new (display);
@@ -348,7 +353,7 @@ pango_x_shutdown_display (Display *display)
 	{
 	  fontmaps = g_list_delete_link (fontmaps, tmp_list);
 	  pango_x_fontmap_cache_clear (xfontmap);
-	  g_object_unref (G_OBJECT (xfontmap));
+	  g_object_unref (xfontmap);
 
 	  return;
 	}
@@ -482,7 +487,7 @@ pango_x_font_map_load_font (PangoFontMap               *fontmap,
 		{
 		  result = (PangoFont *)xfont;
 
-		  g_object_ref (G_OBJECT (result));
+		  g_object_ref (result);
 		  if (xfont->in_cache)
 		    pango_x_fontmap_cache_remove (fontmap, xfont);
 		  
@@ -541,7 +546,7 @@ pango_x_real_get_coverage_win (Display *display)
   int format;
   gulong n_items;
   gulong bytes_after;
-  Atom *data;
+  guchar *data;
   Window retval = None;
   int (*old_handler) (Display *, XErrorEvent *);
   
@@ -555,12 +560,12 @@ pango_x_real_get_coverage_win (Display *display)
 		      0, 4,
 		      False, XA_WINDOW,
 		      &type, &format, &n_items, &bytes_after,
-		      (guchar **)&data);
+		      &data);
   
   if (type == XA_WINDOW)
     {
       if (format == 32 && n_items == 1 && bytes_after == 0)
-	retval = *data;
+	retval = *(Atom *)data;
       
       XFree (data);
     }
@@ -573,11 +578,11 @@ pango_x_real_get_coverage_win (Display *display)
 			  0, 4,
 			  False, XA_WINDOW,
 			  &type, &format, &n_items, &bytes_after,
-			  (guchar **)&data) == Success &&
+			  &data) == Success &&
       type == XA_WINDOW)
     {
       if (format != 32 || n_items != 1 || bytes_after != 0 ||
-	  *data != retval)
+	  *(Atom *)data != retval)
 	retval = None;
       
       XFree (data);
@@ -1366,10 +1371,10 @@ pango_x_fontmap_cache_add (PangoFontMap    *fontmap,
   if (xfontmap->freed_fonts->length == MAX_FREED_FONTS)
     {
       PangoXFont *old_font = g_queue_pop_tail (xfontmap->freed_fonts);
-      g_object_unref (G_OBJECT (old_font));
+      g_object_unref (old_font);
     }
 
-  g_object_ref (G_OBJECT (xfont));
+  g_object_ref (xfont);
   g_queue_push_head (xfontmap->freed_fonts, xfont);
   xfont->in_cache = TRUE;
 }
@@ -1392,7 +1397,7 @@ pango_x_fontmap_cache_remove (PangoFontMap    *fontmap,
   xfontmap->freed_fonts->length--;
   xfont->in_cache = FALSE;
 
-  g_object_unref (G_OBJECT (xfont));
+  g_object_unref (xfont);
 }
 
 static void
@@ -1523,7 +1528,6 @@ pango_x_face_get_coverage (PangoXFace      *xface,
   PangoXFont *xfont;
   PangoXFontMap *xfontmap = NULL; /* Quiet gcc */
   PangoCoverage *result = NULL;
-  GHashTable *coverage_hash;
   Atom atom = None;
 
   if (xface)
@@ -1549,43 +1553,24 @@ pango_x_face_get_coverage (PangoXFace      *xface,
 
   if (!result)
     {
-      guint32 ch;
       PangoMap *shape_map;
-      PangoCoverage *coverage;
-      PangoCoverageLevel font_level;
-      PangoMapEntry *map_entry;
-      
+      PangoEngineShape *engine;
+      gunichar wc;
+
       result = pango_coverage_new ();
       
-      coverage_hash = g_hash_table_new (g_str_hash, g_str_equal);
-      
       shape_map = pango_x_get_shaper_map (language);
+      engine = (PangoEngineShape *)pango_map_get_engine (shape_map, PANGO_SCRIPT_COMMON);
       
-      for (ch = 0; ch < 65536; ch++)
+      for (wc = 0; wc < 65536; wc++)
 	{
-	  map_entry = pango_map_get_entry (shape_map, ch);
-	  if (map_entry->info)
-	    {
-	      coverage = g_hash_table_lookup (coverage_hash, map_entry->info->id);
-	      if (!coverage)
-		{
-		  PangoEngineShape *engine = (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
-		  coverage = engine->get_coverage (font, language);
-		  g_hash_table_insert (coverage_hash, map_entry->info->id, coverage);
-		}
-	  
-	      font_level = pango_coverage_get (coverage, ch);
-	      if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
-		font_level = PANGO_COVERAGE_APPROXIMATE;
-	      
-	      if (font_level != PANGO_COVERAGE_NONE)
-		pango_coverage_set (result, ch, font_level);
-	    }
+	  PangoCoverageLevel level;
+
+	  level = _pango_engine_shape_covers (engine, font, language, wc);
+	  if (level != PANGO_COVERAGE_NONE)
+	    pango_coverage_set (result, wc, level);
 	}
       
-      g_hash_table_foreach (coverage_hash, free_coverages_foreach, NULL);
-      g_hash_table_destroy (coverage_hash);
-
       if (atom)
 	pango_x_store_cached_coverage (xfontmap, atom, result);
     }
